@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2008  Red Hat, Inc.
+# Copyright (C) 2007-2011  Red Hat, Inc.
 # This file is part of python-fedora
 # 
 # python-fedora is free software; you can redistribute it and/or
@@ -36,33 +36,30 @@ except ImportError:
 from turbogears import config, identity
 from turbogears.identity import set_login_attempted
 import cherrypy
+from kitchen.pycompat24 import sets
+from kitchen.text.converters import to_bytes
 
-from fedora.client import BaseClient, AccountSystem, FedoraServiceError
-from fedora import _, __version__
+sets.add_builtin_set()
+
+from fedora.client import AccountSystem, AuthError, BaseClient, \
+        FedoraServiceError
+from fedora import b_, __version__
 
 import logging
 log = logging.getLogger('turbogears.identity.jsonfasprovider')
 
-try:
-    # pylint: disable-msg=W0104
-    # :W0104: we need to figure out if we have a native set type
-    set, frozenset
-except NameError:
-    # :W0622: We need a set type on earlier pythons.
-    from sets import Set as set # pylint: disable-msg=W0622
-    from sets import ImmutableSet as frozenset # pylint: disable-msg=W0622
-
 if config.get('identity.ssl', False):
     fas_user = config.get('fas.username', None)
     fas_password = config.get('fas.password', None)
     if not (fas_user and fas_password):
         raise identity.IdentityConfigurationException(
-                _('Cannot enable ssl certificate auth via identity.ssl without'
-                ' setting fas.usernamme and fas.password for authorization'))
+                b_('Cannot enable ssl certificate auth via identity.ssl'
+                    ' without setting fas.usernamme and fas.password for'
+                    ' authorization'))
     __url = config.get('fas.url', None)
     if __url:
         fas = AccountSystem(__url, username=config.get('fas.username'),
-                password=config.get('fas.password'))
+                password=config.get('fas.password'), retries=3)
 
 
 class JsonFasIdentity(BaseClient):
@@ -98,7 +95,8 @@ class JsonFasIdentity(BaseClient):
         super(JsonFasIdentity, self).__init__(self.fas_url,
                 useragent=self.useragent, debug=debug,
                 username=username, password=password,
-                session_id=session_id, cache_session=self.cache_session)
+                session_id=session_id, cache_session=self.cache_session,
+                retries=3)
 
         if self.debug:
             import inspect
@@ -124,8 +122,8 @@ class JsonFasIdentity(BaseClient):
             self.visit_key = self.session_id
             cherrypy.response.simple_cookie[self.cookie_name] = self.visit_key
         self.log.debug('leaving jsonfas send_request')
-        return super(JsonFasIdentity, self).send_request(method, req_params,
-                auth)
+        return super(JsonFasIdentity, self).send_request(method,
+                req_params=req_params, auth=auth, retries=3)
 
     def __retrieve_user(self):
         '''Attempt to load the user from the visit_key.
@@ -137,8 +135,21 @@ class JsonFasIdentity(BaseClient):
             caller = inspect.getouterframes(inspect.currentframe())[2][3]
             self.log.debug('JSONFASPROVIDER.send_request caller: %s' % caller)
 
+        # The cached value can be in four states:
+        # Holds a user: we successfully retrieved it last time, return it
+        # Holds None: we haven't yet tried to retrieve a user, do so now
+        # Holds a session_id that is the same as our session_id, we unsuccessfully
+        # tried to retrieve a session with this id already, return None
+        # Holds a session_id that is different than the current session_id:
+        # we tried with a previous session_id; try again with the new one.
         if self._retrieved_user:
-            return self._retrieved_user
+            if isinstance(self._retrieved_user, basestring):
+                if self._retrieved_user == self.session_id:
+                    return None
+                else:
+                    self._retrieved_user = None
+            else:
+                return self._retrieved_user
         # I hope this is a safe place to double-check the SSL variables.
         # TODO: Double check my logic with this - is it unnecessary to
         # check that the username matches up?
@@ -152,19 +163,23 @@ class JsonFasIdentity(BaseClient):
             except Exception, e: # pylint: disable-msg=W0703
                 # :W0703: Any errors have to result in no user being set.  The
                 # rest of the framework doesn't know what to do otherwise.
-                self.log.warning(_('jsonfasprovider, ssl, returned errors'
-                    ' from send_request: %s') % e)
+                self.log.warning(b_('jsonfasprovider, ssl, returned errors'
+                    ' from send_request: %s') % to_bytes(e))
                 person = None
             self._retrieved_user = person or None
             return self._retrieved_user
         # pylint: disable-msg=W0702
         try:
             data = self.send_request('user/view', auth=True)
+        except AuthError, e:
+            # Failed to login with present credentials
+            self._retrieved_user = self.session_id
+            return None
         except Exception, e: # pylint: disable-msg=W0703
             # :W0703: Any errors have to result in no user being set.  The rest
             # of the framework doesn't know what to do otherwise.
-            self.log.warning(_('jsonfasprovider returned errors from'
-                ' send_request: %s') % e)
+            self.log.warning(b_('jsonfasprovider returned errors from'
+                ' send_request: %s') % to_bytes(e))
             return None
         # pylint: enable-msg=W0702
 
@@ -386,8 +401,8 @@ class JsonFasIdentityProvider(object):
             user = JsonFasIdentity(visit_key, username=user_name,
                     password=password, using_ssl=using_ssl)
         except FedoraServiceError, e:
-            self.log.warning(_('Error logging in %(user)s: %(error)s') % {
-                'user': user_name, 'error': e})
+            self.log.warning(b_('Error logging in %(user)s: %(error)s') % {
+                'user': to_bytes(user_name), 'error': to_bytes(e)})
             return None
 
         return user
