@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2009  Red Hat, Inc.
+# Copyright (C) 2008-2011  Red Hat, Inc.
 # This file is part of python-fedora
 #
 # python-fedora is free software; you can redistribute it and/or
@@ -27,24 +27,28 @@ repoze.who plugin to authenticate against hte Fedora Account System
 '''
 import os
 import sys
-import webob
+from urllib import quote_plus
 import logging
+
 import pkg_resources
 
 from beaker.cache import Cache
-from fedora.client import AuthError
-from fedora.client.fasproxy import FasProxyClient
+from bunch import Bunch
+from kitchen.text.converters import to_bytes
 from paste.httpexceptions import HTTPFound
 from repoze.who.middleware import PluggableAuthenticationMiddleware
 from repoze.who.classifiers import default_request_classifier
 from repoze.who.classifiers import default_challenge_decider
 from repoze.who.interfaces import IChallenger, IIdentifier
+from repoze.who.plugins.basicauth import BasicAuthPlugin
 from repoze.who.plugins.friendlyform import FriendlyFormPlugin
 from paste.request import parse_dict_querystring, parse_formvars
-from urllib import quote_plus
+import webob
 
+from fedora import _, b_
+from fedora.client import AuthError
+from fedora.client.fasproxy import FasProxyClient
 from fedora.wsgi.csrf import CSRFMetadataProvider, CSRFProtectionMiddleware
-from fedora import _
 
 log = logging.getLogger(__name__)
 
@@ -53,12 +57,40 @@ FAS_CACHE_TIMEOUT = 900 # 15 minutes (FAS visits timeout after 20)
 
 fas_cache = Cache('fas_repozewho_cache', type='memory')
 
-def make_faswho_middleware(app, log_stream, login_handler='/login_handler',
-        login_form_url='/login', logout_handler='/logout_handler',
-        post_login_url='/post_login', post_logout_url=None):
+def fas_request_classifier(environ): 
+    classifier = default_request_classifier(environ) 
+    if classifier == 'browser': 
+        request = webob.Request(environ) 
+        if not request.accept.best_match(['application/xhtml+xml','text/html']): 
+            classifier = 'app' 
+    return classifier 
 
-    faswho = FASWhoPlugin(FAS_URL)
-    csrf_mdprovider = CSRFMetadataProvider(login_handler=login_handler)
+def make_faswho_middleware(app, log_stream=None, login_handler='/login_handler',
+        login_form_url='/login', logout_handler='/logout_handler',
+        post_login_url='/post_login', post_logout_url=None, fas_url=FAS_URL,
+        insecure=False):
+    '''
+    :app: WSGI app that is being wrapped
+    :log_stream: :class:`logging.Logger` to log auth messages
+    :login_handler: URL where the login form is submitted
+    :login_form_url: URL where the login form is displayed
+    :logout_handler: URL where the logout form is submitted
+    :post_login_url: URL to redirect the user to after login
+    :post_logout_url: URL to redirect the user to after logout
+    :fas_url: Base URL to the FAS server
+    :insecure: Allow connecting to a fas server without checking the server's
+        SSL certificate.  Opens you up to MITM attacks but can be useful
+        when testing.  *Do not enable this in production*
+    '''
+
+    # Because of the way we override values (via a dict in AppConfig), we
+    # need to make this a keyword arg and then check it here to make it act
+    # like a positional arg.
+    if not log_stream:
+        raise TypeError('log_stream must be set when calling make_fasauth_middleware()')
+
+    faswho = FASWhoPlugin(fas_url, insecure)
+    csrf_mdprovider = CSRFMetadataProvider()
 
     form = FriendlyFormPlugin(login_form_url,
                               login_handler,
@@ -70,9 +102,11 @@ def make_faswho_middleware(app, log_stream, login_handler='/login_handler',
     form.classifications = { IIdentifier: ['browser'],
                              IChallenger: ['browser'] } # only for browser
 
-    identifiers = [('form', form), ('fasident', faswho)]
+    basicauth = BasicAuthPlugin('repoze.who')
+
+    identifiers = [('form', form), ('fasident', faswho), ('basicauth', basicauth)]
     authenticators = [('fasauth', faswho)]
-    challengers = [('form', form)]
+    challengers = [('form', form), ('basicauth', basicauth)]
     mdproviders = [('fasmd', faswho), ('csrfmd', csrf_mdprovider)]
 
     if os.environ.get('FAS_WHO_LOG'):
@@ -85,19 +119,19 @@ def make_faswho_middleware(app, log_stream, login_handler='/login_handler',
             authenticators,
             challengers,
             mdproviders,
-            default_request_classifier,
+            fas_request_classifier,
             default_challenge_decider,
             log_stream = log_stream,
             )
 
     return app
 
-
 class FASWhoPlugin(object):
 
-    def __init__(self, url, session_cookie='tg-visit'):
+    def __init__(self, url, insecure=False, session_cookie='tg-visit'):
         self.url = url
-        self.fas = FasProxyClient(url)
+        self.insecure = insecure
+        self.fas = FasProxyClient(url, insecure=insecure)
         self.session_cookie = session_cookie
         self._session_cache = {}
         self._metadata_plugins = []
@@ -107,7 +141,7 @@ class FASWhoPlugin(object):
             self._metadata_plugins.append(entry.load())
 
     def keep_alive(self, session_id):
-        log.info(_('Keep alive cache miss'))
+        log.info(b_('Keep alive cache miss'))
         try:
             linfo = self.fas.get_user_info({'session_id': session_id})
         except AuthError, e:
@@ -121,15 +155,15 @@ class FASWhoPlugin(object):
         return linfo
 
     def identify(self, environ):
-        log.info(_('in identify()'))
+        log.info(b_('in identify()'))
         req = webob.Request(environ)
         cookie = req.cookies.get(self.session_cookie)
 
         if cookie is None:
             return None
 
-        log.info(_('Request identify for cookie %(cookie)s') %
-                {'cookie': cookie})
+        log.info(b_('Request identify for cookie %(cookie)s') %
+                {'cookie': to_bytes(cookie)})
         linfo = fas_cache.get_value(key=cookie + '_identity',
                                     createfunc=lambda: self.keep_alive(cookie),
                                     expiretime=FAS_CACHE_TIMEOUT)
@@ -152,7 +186,7 @@ class FASWhoPlugin(object):
             return None
 
     def remember(self, environ, identity):
-        log.info(_('In remember()'))
+        log.info(b_('In remember()'))
         req = webob.Request(environ)
         result = []
 
@@ -165,15 +199,15 @@ class FASWhoPlugin(object):
         return None
 
     def forget(self, environ, identity):
-        log.info(_('In forget()'))
+        log.info(b_('In forget()'))
         # return a expires Set-Cookie header
         req = webob.Request(environ)
 
         linfo = environ.get('FAS_LOGIN_INFO')
         if isinstance(linfo, tuple):
             session_id = linfo[0]
-            log.info(_('Forgetting login data for cookie %(s_id)s') %
-                    {'s_id': session_id})
+            log.info(b_('Forgetting login data for cookie %(s_id)s') %
+                    {'s_id': to_bytes(session_id)})
 
             self.fas.logout(session_id)
 
@@ -189,7 +223,7 @@ class FASWhoPlugin(object):
 
     # IAuthenticatorPlugin
     def authenticate(self, environ, identity):
-        log.info(_('In authenticate()'))
+        log.info(b_('In authenticate()'))
         try:
             login = identity['login']
             password = identity['password']
@@ -213,7 +247,7 @@ class FASWhoPlugin(object):
             user_data = self.fas.get_user_info({'username': login,
                 'password': password})
         except AuthError, e:
-            log.info(_('Authentication failed, setting error'))
+            log.info(b_('Authentication failed, setting error'))
             log.warning(e)
             err = 1
             environ['FAS_AUTH_ERROR'] = err
@@ -234,8 +268,8 @@ class FASWhoPlugin(object):
                 environ['CSRF_AUTH_SESSION_ID'] = environ['FAS_LOGIN_INFO'][0]
                 return login
 
-        err = _('An unknown error happened when trying to log you in.  Please'
-                ' try again.')
+        err = _(u'An unknown error happened when trying to log you in.'
+                ' Please try again.')
         environ['FAS_AUTH_ERROR'] = err
         err_app = HTTPFound(err_goto + '?' + 'came_from=' + came_from)
                             #'&ec=login_err.UNKNOWN_AUTH_ERROR')
@@ -245,7 +279,7 @@ class FASWhoPlugin(object):
         return None
 
     def get_metadata(self, environ):
-        log.info(_('Metadata cache miss - refreshing metadata'))
+        log.info(b_('Metadata cache miss - refreshing metadata'))
         info = environ.get('FAS_LOGIN_INFO')
         identity = {}
 
@@ -276,28 +310,45 @@ class FASWhoPlugin(object):
         return identity
 
     def add_metadata(self, environ, identity):
-        log.info(_('In add_metadata'))
+        log.info(b_('In add_metadata'))
         req = webob.Request(environ)
 
         if identity.get('error'):
-            log.info(_('Error exists in session, no need to set metadata'))
+            log.info(b_('Error exists in session, no need to set metadata'))
             return 'error'
 
         cookie = req.cookies.get(self.session_cookie)
 
         if cookie is None:
-            # @@ Should we resort to this?
-            #cookie = environ.get('CSRF_AUTH_SESSION_ID')
-            return None
+            cookie = environ.get('CSRF_AUTH_SESSION_ID')
+            if cookie is None:
+                return None
 
-        log.info(_('Request metadata for cookie %(cookie)s') %
-                {'cookie':cookie})
+        log.info(b_('Request metadata for cookie %(cookie)s') %
+                {'cookie': to_bytes(cookie)})
         info = fas_cache.get_value(key=cookie + '_metadata',
                 createfunc=lambda: self.get_metadata(environ),
                 expiretime=FAS_CACHE_TIMEOUT)
 
+        #### FIXME: Deprecate this!!!
+        # If we make a new version of fas.who middleware, get rid of saving
+        # user information directly into identity.  Instead, save it into
+        # user, as is done below
         identity.update(info)
 
+        identity['userdata'] = info
+        identity['user'] = Bunch()
+        identity['user'].created = info['creation']
+        identity['user'].display_name = info['human_name']
+        identity['user'].email_address = info['email']
+        identity['user'].groups = info['groups']
+        identity['user'].password = None
+        identity['user'].permissions = info['permissions']
+        identity['user'].user_id = info['id']
+        identity['user'].user_name = info['username']
+        identity['groups'] = info['groups']
+        identity['permissions'] = info['permissions']
+
         if 'repoze.what.credentials' not in environ:
             environ['repoze.what.credentials'] = {}
 
